import react from 'react';

import { TileKeyObject } from './types';

import { CoordinateSystem } from './LiveMap';
import { range } from 'lodash';
import tileUri from './uris';
import TiledLayer from './TiledLayer';

export interface LayerStackProperties {
  /**
   * A key identifying the top left tile
   */
  keyObject: TileKeyObject;
  /**
   * Number of {@link components/map/TiledLayer!TiledLayer}.
   */
  numberOfTiledLayers?: number;
  /**
   * The coordinates system
   */
  coordinateSystem: CoordinateSystem;
}

/**
 *
 * @param props
 * @returns A stack of layers embedded in an SVG element
 *
 * This component does the conversion between the {@link components/map/LiveMap!CoordinateSystem} and
 * the `<`{@link components/map/TiledLayer!TiledLayer}`>`
 * components which units are in tiles.
 *
 * When more then one {@link components/map/TiledLayer!TiledLayer} is required, the tiled layer identified by the keyObject is considered active and new tiles are added
 * as needed and the other layers are used as backups while the tiles are loading.
 *
 */
export const LayerStack: react.FC<LayerStackProperties> = (
  props: LayerStackProperties
) => {
  const numberOfTiledLayers =
    props.numberOfTiledLayers === undefined ? 1 : props.numberOfTiledLayers;

  const activeTiledLayer = Math.floor(numberOfTiledLayers / 2);

  const getTiledLayer = (i: number) => {
    const relativeZoomLevel = i - activeTiledLayer;
    const zoom = 2 ** relativeZoomLevel;
    const origin = {
      x: props.keyObject.x * zoom,
      y: props.keyObject.y * zoom,
    };
    const keyObject = {
      provider: props.keyObject.provider,
      zoomLevel: props.keyObject.zoomLevel + relativeZoomLevel,
      x: Math.floor(origin.x),
      y: Math.floor(origin.y),
    };
    const shift = {
      x: -origin.x,
      y: -origin.y,
    };
    const key = tileUri({
      provider: keyObject.provider,
      zoomLevel: keyObject.zoomLevel,
    });

    return (
      <TiledLayer
        key={key}
        keyObject={keyObject}
        shift={shift}
        zoom={256 / zoom}
        coordinateSystem={props.coordinateSystem}
      />
    );
  };

  //  console.log(`tiledLayers: ${JSON.stringify(tiledLayers)}`);

  return (
    <svg width='100%' height='100%' data-testid='layer-stack'>
      <g
        transform={`translate(${props.coordinateSystem.shift.x}, ${props.coordinateSystem.shift.y}) scale(${props.coordinateSystem.zoom})`}
      >
        {
          // Tiled layers with less detail
          range(0, activeTiledLayer).map(getTiledLayer)
        }
        {
          // Tiled layers with more details
          range(numberOfTiledLayers - 1, activeTiledLayer, -1).map(
            getTiledLayer
          )
        }
        {
          // And the active one
          getTiledLayer(activeTiledLayer)
        }
      </g>
    </svg>
  );
};

export default LayerStack;
