import react, { useRef } from 'react';
import TileSet from './TileSet';
import { Point, TileKeyObject } from './types';
import { CoordinateSystem } from './LiveMap';

export interface TiledLayerProperties {
  /**
   * A key identifying the top left tile
   */
  keyObject: TileKeyObject;
  /**
   * The translation to apply.
   */
  shift: Point;
  /**
   * The zoom to apply. If equal to 256 (the tile size), the layer is considered active and should add tiles which are in its viewport
   */
  zoom: number;
  /**
   * The coordinate system
   */
  coordinateSystem: CoordinateSystem;
}

/**
 *
 * @param props
 * @returns A layer of tiles.
 * This component wraps a `<TileSet>` in an SVG `<g>` element taking care of the scale and translation.
 *
 */
export const TiledLayer: react.FC<TiledLayerProperties> = (
  props: TiledLayerProperties
) => {
  const g = useRef<SVGGElement>(null);
  var viewPort: any = undefined;
  if (
    props.zoom === 256 &&
    g.current !== null &&
    g.current.ownerSVGElement !== null &&
    g.current.ownerSVGElement.parentElement !== null
  ) {
    const nearerHTMLParent = g.current.ownerSVGElement.parentElement;
    viewPort = {
      topLeft: {
        x:
          props.keyObject.x +
          Math.floor(
            -props.coordinateSystem.shift.x / props.coordinateSystem.zoom / 256
          ),
        y:
          props.keyObject.y +
          Math.floor(
            -props.coordinateSystem.shift.y / props.coordinateSystem.zoom / 256
          ),
      },
      bottomRight: {
        x:
          props.keyObject.x +
          Math.ceil(
            (-props.coordinateSystem.shift.x + nearerHTMLParent.offsetWidth) /
              props.coordinateSystem.zoom /
              256
          ) -
          1,
        y:
          props.keyObject.y +
          Math.ceil(
            (-props.coordinateSystem.shift.y + nearerHTMLParent.offsetHeight) /
              props.coordinateSystem.zoom /
              256
          ) -
          1,
      },
    };
  }
  return (
    <g
      transform={`scale(${props.zoom}) translate(${props.shift.x}, ${props.shift.y})`}
      ref={g}
    >
      <TileSet
        keyObject={{
          provider: props.keyObject.provider,
          zoomLevel: props.keyObject.zoomLevel,
          x: 0,
          y: 0,
        }}
        viewPort={viewPort}
      />
    </g>
  );
};

export default TiledLayer;
